local LOG = "game.log"
tUnit = {}
tUnit.__index = tUnit
--None of the inputs are typesafed, this is because all data is assumed right.
--Requires table with {iLevel, iHP, iMP, iStr, iAgi, iSpr, iArmor, iArmorPen, sName, pGraphic, pFace, aVoice, x, y, map, bMoves}
function tUnit:new (t)
	local t = t or {}
	setmetatable(t, self)
	self.level = t[iLevel]
    self.MaxHP = t[iHP]
    self.HP = t[iHP]
    self.MaxMP = t[iMP]
    selfMP =  t[iMP]
    selfxp = t[iXP]
    self.strength = t[iStr]
    self.agility = t[iAgi]
    self.spirit = t[iSpr]
    self.armor = t[iArmor]
    self.armorPen = t[iArmorPen]
    self.name = t[sName]
    self.graphic = t[pGraphic]
    self.face = t[pFace]
    self.voice= t[aVoice]
    self.alive = true
	if t[bMoves] then
		self.x = t[x]
		self.y = t[y]
		self.map = t[map]
		self.moves = true
	else
		self.moves = false
	end
    if DEBUG_MODE and DEBUG_LEVEL >= 1 then
        DEBUG_Print(LOG, " Created new unit " .. self["name"] .. ".")
    end
    return t
end

function tUnit:SetVoice(pPicture)
    self.voice = pPicture
end

function tUnit:Move(sDirection, iFlagMove, iMap, x, y)
	--[[
		iFlagMove has 2 possible choices.
		0 = normal keyboard move
		1 = functional move (teleport), used for switching maps, and other functions
		that need teleportation.
	--]]
	
	if self.moves then
		if iCount == 0 and iMap == self.map then
			if sDirection == 'up' then
				self.y = self.y + 1
			elseif sDirection == 'down' then
				self.y = self.y - 1
			elseif sDirection == 'left' then
				self.y = self.y - 1
			else
				self.y = self.y + 1
			end
		else
			self.y = x
			self.y = y
			self.y = map
		end
	else
		DEBUG_Print(LOG, "Immobile Unit attempted to move. Unit Name: " .. self.name ".")
	end
end

function tUnit:SetFace(pPicture)
    self.face = pPicture
end

function tUnit:SetGraphic(pPicture)
    self.graphic = pPicture
end

function tUnit:ModifyHeath(ivalue)
    local value = self.HP
    if ivalue < 0 then
        if self.HP - ivalue < 0 then
            self.HP = 0
            self.alive = false
        else
            self.HP = self.HP + ivalue
        end
    else
        if self[HP] + ivalue > self[MaxHP] then
            self[HP] = self[MaxHP]
        else
            self[HP] = self[HP] + ivalue
        end
    end
    if DEBUG_MODE and DEBUG_VALUE == 3 then
        DEBUG_Print(LOG, os.date() .. " Unit: " .. self[name] .. "'s health became " .. self[HP] .. " from " .. value .. ".")
    end
end

function tUnit:ModifyMana(ivalue)
    local value = self[MP]
    if ivalue < 0 then
        if self[MP] - ivalue < 0 then
            self[MP] = 0
        else
            self[MP] = self[MP] + ivalue
        end
    else 
        if self[MP] + ivalue > self[MaxMP] then
            self[MP] = self[MaxMP]
        else
            self[MP] = self[MP] + ivalue
        end
    end
    if DEBUG_MODE and DEBUG_VALUE == 3 then
        DEBUG_Print(LOG, os.date() .. " Unit: " .. self[name] .. "'s mana became " .. self[MP] .. " from " .. value .. ".")
    end
end

function tUnit:ModifyLevel(ivalue)
    local value = self[level]
    if ivalue < 0 then
        if self[level] - ivalue < 1 then
            self[level] = 1
        else
            self[level] = self[level] + ivalue
        end
    else
        if self[level] + ivalue > MAX_LEVEL then
            self[level] = MAX_LEVEL
        else
            self[level] = self[level] + ivalue
        end
    end
    if DEBUG_MODE and DEBUG_LEVEL >= 2 then
        DEBUG_Print(LOG, os.date() .. "Unit: " .. self[name] .. "'s level became " .. self[level] .. " from " .. value .. ".")
    end
end


--currently don't need to deduct xp, why would it be?
function tUnit:AddXP(ivalue, ilevel)
    if ivalue > XPNEEDED[ilevel] then
        self[xp] = 0
        self.ModifyLevel(1)
    else
        self[xp] = self[xp] + ivalue
    end
end

function tUnit:ModifyArmor(ivalue)
    if ivalue < 0 then
        if self[armor] + ivalue < -MAX_ARMOR then
            self[armor] = -MAX_ARMOR
        else
            self[armor] = self[armor] + ivalue
        end
    else
        if self[armor] + ivalue > MAX_ARMOR then
            self[armor] = MAX_ARMOR
        else
            self[armor] = self[armor] + ivalue
        end
    end
end